#!/usr/bin/perl

=head1 script perl2xml.pl

=head2 AUTEUR : Schaeffer Raphael 

=head2 Date : JANVIER 2006

=head2 USAGE : perl2xml fichierInput

=head2 Descriptif

Transforme un fichier perl en fichier XML, pour l'afficher à l'aide d'une XSLT sur un site internet.
Ici le but est de pouvoir facilement générer un index des fonctions avec une feuille XSL.
Le fichier en sortie porte le même nom que celui en entrée au nom de l'extension près( .pl -> .xml )

Peut être améliorer pour prendre en compte la doc et les commentaires, mais demande la conception d'un parser perl qui prend en compte les changements de contexte. Le nombre de noeud augamentant il serait envisageable d'utiliser un créateur DOM.

=head2 Allure du fichier XML en sortie

<zone=type="codePerl">
<type>module|script</type>
<main>
	<nom>...<nom>
	<ligne><![CDATA[...  ...]]></ligne>....
	</main>
<fonction>
	<nom>...</nom>
	<code>
	<ligne><![CDATA[.... ....]]></ligne>....
	<code>
</fonction>
<fonction>
.
.
.
</fonction>
</zone>

=cut

use strict;
use warnings;
use diagnostics;
use erreur;
use utile;


#----------main

my $input=shift @ARGV;
my $ext=utile::getExtension($input);
my $output=utile::changeExtension($input,"xml");
# donne type : module ou script
my $type;
if($ext eq "pl"){
	$type="script";
}elsif($ext eq "pm"){
	$type="module";
}else{
	erreur::affiche("extension",$input);
}

#ouverture input et output
open(HDIN,"<:utf8",$input) or erreur::affiche("ouvertureF",$input);
open(HDOUT,">:utf8",$output) or erreur::affiche("ouvertureF",$output);


# en-tête
print HDOUT "<codePerl><type>$type</type>\n\t<main><nom>$input</nom>\n\t<code><ligne><![CDATA[";

# copie main
while(1){
	my $ligne=<HDIN>;
	# si le script est terminé sans aucune fonction
	if (!defined $ligne){
		&finMain;
		&finPerl;
		exit;
	}
	# si le main est terminé avec une définition de fonction
	my $nomFonction=estDebutFonction($ligne);
	if (defined $nomFonction){
		&finMain;
		ecrireFonction($nomFonction,$ligne);
		last;
	}
	#sinon on recopie le contenu du main et on recommence
	print HDOUT "]]></ligne>\n<ligne><![CDATA[$ligne";
}
my $html=utile::changeExtension($output,"html");
system(xsltproc -o $html codePerl.xsl $output);
exit;

#---------------fonction

# fermeture des balise
sub finFonction{
	print HDOUT "]]></ligne>\n\t\t</code>\n\t</fonction>\n";
	return;
}

sub finMain{
	print HDOUT "]]></ligne>\n\t</code>\t\n</main>\n";
	return;
}

sub finPerl{
	print HDOUT "</codePerl>";
	close HDIN;
	close HDOUT;
	return;
}


# indique si la ligne correspond �un en-tête de fonction
# dans les bons cas renvoie le nom de la fonction
# sinon undef
sub estDebutFonction{
	my $ligne=shift;
	$ligne=~/^\s*&?sub\s+([\w0-9_]+)/;
	return $1;
}

# met une balise et recopie la fonction
sub ecrireFonction{
	my($nomFonction,$ligne)=@_;
# 	en-tête de la fonction
	print HDOUT "\t<fonction>\n\t\t<nom>$nomFonction</nom>\n\t\t<code>\n<ligne><![CDATA[$ligne";
	while(1){
		$ligne=<HDIN>;
		# si le script est terminé donc la dernière fonction aussi
		if (!defined $ligne){
			&finFonction;
			&finPerl;
			exit;
		}
		# si la définition de la fonction est terminé et qu'il y en a une suivante
		if(my $nomFonction=estDebutFonction($ligne)){
			&finFonction;
			ecrireFonction($nomFonction,$ligne);
			return;
		}
		# sinon on recopie la défintion de la fonction et on recommence
		print HDOUT "]]></ligne>\n<ligne><![CDATA[$ligne";
	}
}